﻿using Core.Config;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Builder;
using System.Runtime.Loader;
using Minio;
using Microsoft.AspNetCore.Authentication.Cookies;
using Core.Global;
using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.IdentityModel.Tokens;
using Core.General;
using Newtonsoft.Json.Serialization;
using Microsoft.AspNetCore.Mvc.Controllers;
using Microsoft.Extensions.Hosting;
using NAutowired;
using Core.Extensions;
using System.Reflection.Emit;

namespace Core
{
    /// <summary>
    /// WebApi 启动器
    /// </summary>
    public sealed class Bootstrap
    {
        private readonly WebApplicationBuilder builder;
        private readonly IWebHostEnvironment environment;
        private readonly IServiceCollection services;
        private readonly IConfiguration configuration;
        private readonly AppSettings settings;

        private bool isDbContext = false;
        private bool isCors = false;
        private bool isSwagger = false;
        private bool isSwaggerOnlyDevelopment = false;

        /// <summary>
        /// 获取IServiceCollection
        /// </summary>
        internal IServiceCollection Services { get => services; }

        public Bootstrap()
        {
            var args = Environment.GetCommandLineArgs();
            var builder = WebApplication.CreateBuilder(args);

            this.builder = builder;
            environment = builder.Environment;
            services = builder.Services;
            configuration = builder.Configuration;
            settings = configuration.Get<AppSettings>();

            services.AddHttpContextAccessor();


            this.AddDependencyInjection()
                .AddControllers()
                .AddAuthentication()
                .AddMinio()
                .AddCors()
                .AddMemoryCache()
                .AddSqlSugar()
                .AddSwaggerUI();

        }

        public Bootstrap(Action<IServiceCollection> setup): this()
        {
            setup(services);
        }

        /// <summary>
        /// 扫描并加载运行目录下的程序集
        /// </summary>
        /// <returns></returns>
        void ScanningDynamicAssemblies()
        {
            var loaded = AppDomain.CurrentDomain.GetAssemblies().Where(p => !p.IsDynamic).ToDictionary(p => p.Location);
            var dlls = Directory.GetFiles(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location) ?? ".", "*.dll");
            foreach (var dll in dlls)
            {
                if (loaded.ContainsKey(dll)) continue;
                try
                {
                    AssemblyLoadContext.Default.LoadFromAssemblyPath(dll);
                    Console.WriteLine($"程序集加载：{dll}");
                }
                catch (Exception)
                {
                    Console.WriteLine($"程序集加载失败：{dll}");
                }
            }
        }


        /// <summary>
        /// 添加依赖注入
        /// </summary>
        /// <returns></returns>
        Bootstrap AddDependencyInjection()
        {
            // 执行程序集扫描
            ScanningDynamicAssemblies();

            var s2 = Assembly.GetExecutingAssembly().GetReferencedAssemblies().Select(Assembly.Load).ToArray();
            var a2 = AppDomain.CurrentDomain.GetAssemblies();

            foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                try
                {
                    var types = assembly.GetTypes();

                    foreach (var type in types)
                    {
                        var di = type.GetCustomAttribute<DependencyInjectionAttribute>();
                        if (di == null || !type.IsClass || type.IsAbstract) continue;
                        services.TryAdd(ServiceDescriptor.Describe(di.AsType ?? type, type, di.Lifetime));
                    }
                }
                catch
                {

                }

            }

            return this;
        }
        /// <summary>
        /// 添加Minio服务
        /// </summary>
        /// <param name="contextLifetime"></param>
        /// <returns></returns>
        Bootstrap AddMinio(ServiceLifetime contextLifetime = ServiceLifetime.Singleton)
        {
            services.TryAdd(new ServiceDescriptor(typeof(IMinioClient), factory =>
            {
                var client = new MinioClient()
                .WithEndpoint(settings.Minio.Endpoint)
                .WithCredentials(settings.Minio.AccessKey, settings.Minio.SecretKey)
                .WithSSL(settings.Minio.Secure)
                .Build();
                return client;
            }, contextLifetime));
            return this;
        }
        /// <summary>
        /// 添加内存缓存
        /// </summary>
        /// <returns></returns>
        Bootstrap AddMemoryCache(Action<MemoryCacheOptions> setupAction = null)
        {
            if (setupAction == null)
                services.AddMemoryCache();
            else
                services.AddMemoryCache(setupAction);
            return this;
        }

        /// <summary>
        /// 添加授权服务
        /// </summary>
        /// <returns></returns>
        Bootstrap AddAuthentication()
        {
            services.AddAuthentication(x =>
            {
                x.DefaultAuthenticateScheme = CookieAuthenticationDefaults.AuthenticationScheme;
                x.DefaultChallengeScheme = CookieAuthenticationDefaults.AuthenticationScheme;
            })
            .AddCookie(x =>
            {
                x.ExpireTimeSpan = TimeSpan.FromHours(6);
                x.Cookie.Name = "user-session";
                x.SlidingExpiration = true;
                x.LoginPath = "/login";
                x.ReturnUrlParameter = "redirect";
            })
            .AddJwtBearer(x =>
            {
                //将配置文件中的相关内容反序列化
                var tokenInfo = settings.JwtConfiguration;
                x.Events = new JwtBearerEvents
                {
                    //验证失败时的处理
                    OnAuthenticationFailed = context =>
                    {
                        //若失败类型为过期，则返回特定Header，便于客户端判断
                        if (context.Exception.GetType() == typeof(SecurityTokenExpiredException))
                        {
                            context.Response.Headers.Add("tokenErr", "expired");
                        }
                        return Task.CompletedTask;
                    },
                    OnMessageReceived = context =>
                    {
                        var accessToken = context.Request.Cookies[XAccess.Token];

                        if (!string.IsNullOrEmpty(accessToken))
                        {
                            context.Token = accessToken;
                        }

                        return Task.CompletedTask;
                    }
                };
                x.RequireHttpsMetadata = false;
                x.SaveToken = true;
                x.TokenValidationParameters = new TokenValidationParameters
                {
                    ValidateIssuerSigningKey = true,
                    IssuerSigningKey = new SymmetricSecurityKey(Encoding.UTF8.GetBytes(tokenInfo.AccessSecret)),
                    ValidAlgorithms = new[] { SecurityAlgorithms.HmacSha256 },
                    ValidIssuer = tokenInfo.Issuer,
                    ValidAudience = tokenInfo.Audience,
                    ValidateIssuer = true,
                    ValidateAudience = true,
                    ValidateLifetime = true,
                    ClockSkew = TimeSpan.FromSeconds(tokenInfo.ClockSkew)
                };
            });
            return this;
        }

        /// <summary>
        /// 添加接口跨域
        /// </summary>
        /// <returns></returns>
        Bootstrap AddCors()
        {
            isCors = true;
            services.AddCors(setup => setup.AddDefaultPolicy(x => x.AllowAnyOrigin().AllowAnyHeader().AllowAnyMethod()));
            //services.addcor(setup => setup.AddDefaultPolicy(x => x.AllowAnyOrigin().AllowAnyHeader().AllowAnyMethod()));
            return this;
        }

        /// <summary>
        /// 添加SwaggerUI帮助文档
        /// </summary>
        /// <returns></returns>
        Bootstrap AddSwaggerUI(bool onlyDevelopment = true)
        {
            isSwaggerOnlyDevelopment = onlyDevelopment;
            isSwagger = true;
            services.AddSwaggerGen(setup =>
            {
                // 为 Swagger JSON and UI设置xml文档注释路径
                var basePath = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);//获取应用程序所在目录（绝对，不受工作目录影响，建议采用此方法获取路径）
                var xmls = Directory.GetFiles(basePath, "*.xml").ToArray();
                foreach (var xmlPath in xmls)
                {
                    setup.IncludeXmlComments(xmlPath);
                }
                setup.SupportNonNullableReferenceTypes();
            });
            return this;
        }

        /// <summary>
        /// 添加控制器
        /// </summary>
        /// <param name="parts"></param>
        /// <returns></returns>
        Bootstrap AddControllers(params Assembly[] parts)
        {
            var mvc = services.AddControllers(opt =>
            {
                opt.Filters.Add<CustomerExceptionFilter>();// 添加自定义异常处理器
                                                           //opt.OutputFormatters.Add(new XmlSerializerOutputFormatterNamespace());
            })
                ;
            mvc.AddNewtonsoftJson(x =>
            {
                //忽略循环引用
                x.SerializerSettings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
                x.SerializerSettings.Converters.Add(new IsoDateTimeConverter());
                x.SerializerSettings.ContractResolver = new DefaultContractResolver();
                x.SerializerSettings.Formatting = Formatting.None;
                x.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
                //x.SerializerSettings.Converters.Add(new StringEnumConverter());
            });
            mvc.AddApplicationPart(typeof(GeneralController).Assembly);
            foreach (var part in parts) mvc.AddApplicationPart(part);
            mvc.
                //AddXmlSerializerFormatters().
                AddControllersAsServices();
            return this;
        }

        /// <summary>
        /// 添加 SqlSugar
        /// </summary>
        /// <returns></returns>
        Bootstrap AddSqlSugar()
        {
            var settings = configuration.AppSettings();
            SnowFlakeSingle.WorkId = settings.WorkId;

            var s = Assembly.GetEntryAssembly().GetReferencedAssemblies();
            services.AddSingleton<ISqlSugarClient>(x =>
            {

                SqlSugarScope sqlSugar = new SqlSugarScope(new ConnectionConfig()
                {
                    DbType = settings.ConnectionDbType,
                    ConnectionString = settings.ConnectionString,
                    IsAutoCloseConnection = true,
                    ConfigureExternalServices = new ConfigureExternalServices
                    {
                        //注意:  这儿AOP设置不能少
                        EntityService = (c, p) =>
                        {
                            /***低版本C#写法***/
                            // int?  decimal?这种 isnullable=true 不支持string(下面.NET 7支持)
                            if (p.IsPrimarykey == false && c.PropertyType.IsGenericType &&
                            c.PropertyType.GetGenericTypeDefinition() == typeof(Nullable<>))
                            {
                                p.IsNullable = true;
                            }

                            /***高版C#写法***/
                            //支持string?和string  
                            if (p.IsPrimarykey == false && new NullabilityInfoContext()
                             .Create(c).WriteState is NullabilityState.Nullable)
                            {
                                p.IsNullable = true;
                            }
                        }
                    }
                },
               db =>
               {
                   //每次上下文都会执行

                   //获取IOC对象不要求在一个上下文
                   //var log=s.GetService<Log>()

                   //获取IOC对象要求在一个上下文
                   //var appServive = s.GetService<IHttpContextAccessor>();
                   //var log= appServive?.HttpContext?.RequestServices.GetService<Log>();

                   db.Aop.OnLogExecuting = (sql, pars) =>
                   {
                       //Console.WriteLine(sql);
                   };
               });

                // 创建数据库
                sqlSugar.DbMaintenance.CreateDatabase();
                // 创建数据库表
                sqlSugar.CodeFirst.SetStringDefaultLength(200).InitTables(Utils.GetEntityTypes());
                // 创建种子数据
                foreach (var type in Utils.GetEntitySeedDataTypes())
                {
                    var seedData = Activator.CreateInstance(type) as dynamic;
                    var hasData = seedData.HasData();
                    sqlSugar.Storageable(hasData).ToStorage().AsInsertable.ExecuteCommand();
                }
                return sqlSugar;
            });
            return this;
        }
        /// <summary>
        /// 运行服务
        /// </summary>
        public void Run()
        {
            // 使用 NAutowired 框架
            services.Replace(ServiceDescriptor.Transient<IControllerActivator, NAutowiredControllerActivator>());

            var app = builder.Build();
            if (isSwaggerOnlyDevelopment)
            {
                if (app.Environment.IsDevelopment()) app.UseSwagger().UseSwaggerUI();
            }
            else if (isSwagger)
            {
                app.UseSwagger().UseSwaggerUI();
            }

            //// 执行数据库迁移
            //if (isDbContext)
            //{
            //    using (var scope = app.Services.CreateScope())
            //    {
            //        var context = scope.ServiceProvider.GetService<DbContext>();
            //        context?.Database.Migrate();
            //    }
            //}
            if (isCors)
            {
                app.UseCors();
            };
            app.UseStaticFiles();
            app.UseRouting();
            app.UseAuthentication();
            app.UseAuthorization();
            app.MapControllers();

            app.Run();
        }
    }
}
