﻿using Core.Controllers.Dto;
using Core.Entitys;
using Core.Extensions;
using Core.General;
using Mapster;
using Microsoft.AspNetCore.Authentication.Cookies;
using Microsoft.AspNetCore.Authentication;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Security.Claims;
using System.Text;
using System.Threading.Tasks;

namespace Core.Controllers
{
    namespace Dto
    {
        /// <summary>
        /// 添加用户信息
        /// </summary>
        public class AddUserInfo
        {
            /// <summary>
            /// 昵称
            /// </summary>
            [MaxLength(32)]
            public virtual string NickName { get; set; } = string.Empty;

            /// <summary>
            /// 头像
            /// </summary>
            [MaxLength(512)]
            public virtual string Avatar { get; set; } = string.Empty;

            /// <summary>
            /// 性别
            /// </summary>
            public virtual SexEnum Sex { get; set; } = SexEnum.Unknown;

            /// <summary>
            /// 手机号码
            /// </summary>
            [MaxLength(11), Phone]
            public virtual string Phone { get; set; } = string.Empty;

            /// <summary>
            /// 备注
            /// </summary>
            public virtual string Remark { get; set; } = string.Empty;
            /// <summary>
            /// 状态
            /// </summary>

            public virtual StatusEnum Status { get; set; }
        }

        /// <summary>
        /// 添加租户信息
        /// </summary>
        public class AddTenantInfo
        {
            /// <summary>
            /// 名称
            /// </summary>
            [MaxLength(32)]
            public virtual string Name { get; set; } = string.Empty;

            /// <summary>
            /// 手机号码
            /// </summary>
            [MaxLength(11), Phone]
            public virtual string Phone { get; set; } = string.Empty;

            /// <summary>
            /// 备注
            /// </summary>
            public virtual string Remark { get; set; } = string.Empty;
        }

        /// <summary>
        /// 用户登录成功后返回
        /// </summary>
        public class UserLoginResult
        {
            /// <summary>
            /// 授权Token
            /// </summary>
            public string AccessToken { get; set; } = string.Empty;
            /// <summary>
            /// 刷新Token
            /// </summary>
            public string RefreshToken { get; set; } = string.Empty;
            /// <summary>
            /// 授权Token失效时间
            /// </summary>
            public DateTime AccessExpiration { get; set; }
            /// <summary>
            /// 刷新Token失效时间
            /// </summary>
            public DateTime RefreshExpiration { get; set; }
        }

        /// <summary>
        /// 用户登录模型（通过账号密码）
        /// </summary>
        /// <remarks>账号是手机号</remarks>
        public class UserLoginByPhone
        {
            /// <summary>
            /// 账号
            /// </summary>
            [Required, MaxLength(11)]
            public string Account { get; set; } = string.Empty;
            /// <summary>
            /// 登录凭证（密码）
            /// </summary>
            [Required, MaxLength(16)]
            public string Password { get; set; } = string.Empty;
        }
    }
    /// <summary>
    /// 系统相关
    /// </summary>
    public class SystemController : GeneralController
    {
        /// <summary>
        /// 根据枚举名称获取枚举选项
        /// </summary>
        /// <param name="enumTypeName">枚举名称</param>
        /// <returns></returns>
        [HttpGet]
        public EnumOption[] GetEnum([FromQuery, Required] string enumTypeName)
        {
            Type enumType = null;


            // 获取当前程序集中所有定义的枚举类型
            foreach (Type type in Utils.GetReferencedAssemblies().SelectMany(x => x.GetTypes()))
            {
                if (type.IsEnum && type.Name == enumTypeName)
                {
                    enumType = type;
                    break;
                }
            }

            if (enumType != null && enumType.IsEnum)
            {
                return enumType.GetEnumOptions();
            }
            else
            {
                this.Logger.LogWarning("找不到名为 " + enumTypeName + " 的枚举类型");
                return Array.Empty<EnumOption>();
            }
        }

        /// <summary>
        /// 记录日志
        /// </summary>
        /// <param name="message"></param>
        /// <param name="level"></param>
        [HttpGet]
        public void Log([FromQuery, Required] string message, [FromQuery] LogLevel level = LogLevel.Information)
        {
            if (string.IsNullOrEmpty(message)) throw new Exception("日志message是必须的");

            this.Logger.Log(level, message);
        }

        /// <summary>
        /// 获取指定用户信息
        /// </summary>
        [HttpGet, HybridAuthorize]
        public async Task<SysUser> GetUserInfoById([FromQuery] string userId)
        {
            var id = userId.TryLong();
            var user = await Db.Queryable<SysUser>().FirstAsync(o => o.Id == id);
            if (user == null) throw new Exception("无效用户");
            return user;
        }

        /// <summary>
        /// 获取当期登录账号用户信息
        /// </summary>
        [HttpGet, HybridAuthorize]
        public async Task<SysUser> GetUserInfo()
        {
            var user = await Db.Queryable<SysUser>().InSingleAsync(CurrentUserId.Value);
            if (user == null) throw new Exception("无效用户");
            return user;
        }


        /// <summary>
        /// 重复手机号检测
        /// </summary>
        /// <param name="phone">手机号</param>
        /// <returns></returns>
        [HttpGet]

        public async Task<bool> DuplicatePhone([FromQuery, Phone(ErrorMessage = "请输入正确的手机号")] string phone)
        {
            var result = await Db.Queryable<SysUser>().AnyAsync(o => o.Phone == phone);

            return result;
        }

        /// <summary>
        /// 获取系统配置
        /// </summary>
        /// <param name="configKey"></param>
        /// <param name="fromCache">从缓存中读取，默认值true</param>
        /// <returns></returns>
        [HttpGet]
        public async Task<SysConfig> GetConfig([FromQuery, Required(ErrorMessage = "configKey 是必须的")] string configKey, [FromQuery] bool fromCache = true)
        {
            var cacheKey = $"{nameof(SysConfig)}:${configKey}";
            // 先清空缓存
            if (!fromCache) { Cache.Remove(cacheKey); }

            var configEntity = await Cache.GetOrCreateAsync(cacheKey, async x =>
            {
                x.SetSlidingExpiration(TimeSpan.FromMinutes(5));
                var entity = await Db.Queryable<SysConfig>().SingleAsync(o => o.Code == configKey);
                return entity;
            });
            return configEntity;
        }


        [HttpPost, HybridAuthorize]
        public async void AddTenant(AddTenantInfo info)
        {
            var duplicate = await DuplicatePhone(info.Phone);
            if(duplicate) throw new Exception($"Duplicate phone: {info.Phone}");

            var org = new SysOrg() { Name = info.Name, Level = 0,  };
            var tenant = new SysTenant() { };
            var user = new SysUser() { NickName = $"租户-{info.Name}", AccountType = AccountTypeEnum.SysAdmin, Remark = info.Remark, Phone = info.Phone };

            var identity = new SysIdentity();
   
        }

        /// <summary>
        /// 添加用户
        /// </summary>
        /// <param name="user"></param>
        /// <returns></returns>
        /// <exception cref="Exception"></exception>
        [HttpPost]
        public async Task<SysIdentity> AddUser(AddUserInfo user)
        {
            var phone = user.Phone;

            // 手机号检测，手机号如果存在，禁止重复创建用户
            if (!string.IsNullOrEmpty(phone))
            {
                var duplicate = await DuplicatePhone(phone);
                if (duplicate) throw new Exception($"Duplicate {phone}");
            }
            // 密码盐
            var salt = Utils.RandomString(12);
            // 默认密码配置
            var password = (await GetConfig("sys_password", false)).Value.Salt(salt);


            var userEntity = user.Adapt<SysUser>();
            var identityEntity = new SysIdentity() { User = userEntity, Identifier = phone, Salt = salt, Credential = password };
            var result = await Db.InsertNav(identityEntity).Include(t => t.User).ExecuteReturnEntityAsync();

            return result;
        }

        /// <summary>
        /// 账户登录
        /// </summary>
        [HttpPost]
        public async Task<UserLoginResult> Login(UserLoginByPhone model)
        {
            var entity = await Db.Queryable<SysIdentity>().Includes(o=>o.User).FirstAsync(o => o.Identifier == model.Account);
            if (entity == null) { throw new Exception("账号或密码输入错误"); }
            if (entity.User.Status != StatusEnum.Enable) { throw new Exception(entity.User.Status.GetDescription()); }
            var user = entity.User;
            if (entity.Credential != model.Password.Salt(entity.Salt)) { throw new Exception("账号或密码输入错误"); }

            var result = InternalLogin(entity, out List<Claim> claims);

            // 添加 Cookie
            var identity = new ClaimsIdentity(claims, CookieAuthenticationDefaults.AuthenticationScheme);
            var principal = new ClaimsPrincipal(identity);
            await HttpContext.SignInAsync(CookieAuthenticationDefaults.AuthenticationScheme, principal);
            Response.Cookies.Append(XAccess.Token, result.AccessToken);

            return result;
        }

        /// <summary>
        /// 退出登录
        /// </summary>
        /// <returns></returns>
        [HttpPost]
        public async Task<ActionResult> Logout()
        {
            await HttpContext.SignOutAsync(CookieAuthenticationDefaults.AuthenticationScheme);
            Response.Cookies.Delete(XAccess.Token);
            return Ok();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="identity"></param>
        /// <param name="claims"></param>
        /// <param name="notExpire">token不过期</param>
        /// <returns></returns>
        UserLoginResult InternalLogin(SysIdentity identity, out List<Claim> claims, bool notExpire = false)
        {
            var user = identity.User;

            var settings = Config.AppSettings();
            //将配置文件中的相关内容反序列化
            var tokenInfo = settings.JwtConfiguration;


            claims = new List<Claim>()
            {
                new Claim(ClaimsExtensions.UserId, user.Id.ToString()),
                new Claim(ClaimsExtensions.UserName, user.NickName),
                //new Claim(ClaimsExtensions.TenantId, $"{user.}"),
                new Claim(ClaimsExtensions.OrgId, $"{user.OrgId}"),
                new Claim(ClaimsExtensions.Super,$"{user.AccountType == AccountTypeEnum.SysAdmin}"),
                new Claim(ClaimsExtensions.Identifier,identity.Identifier),
            };
            var AccessToken = claims.CreateJwtToken(tokenInfo.AccessSecret, tokenInfo.Issuer, tokenInfo.Audience, notExpire ? int.MaxValue : tokenInfo.AccessExpiration, out DateTime AccessExpiration);
            var RefreshToken = claims.CreateJwtToken(tokenInfo.RefreshSecret, tokenInfo.Issuer, tokenInfo.Audience, notExpire ? int.MaxValue : tokenInfo.RefreshExpiration, out DateTime RefreshExpiration);

            return new UserLoginResult() { AccessToken = AccessToken, RefreshToken = RefreshToken, AccessExpiration = AccessExpiration, RefreshExpiration = RefreshExpiration };
        }
    }
}
