﻿using Microsoft.AspNetCore.Routing;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Core
{
    #region 数据库实体相关


    /// <summary>
    /// 实体
    /// </summary>
    public interface IEntity
    {
    }
    /// <summary>
    /// 软删除接口
    /// </summary>
    public interface IDeleted
    {
        /// <summary>
        /// 软删除
        /// </summary>
        public DateTime? DeleteTime { get; set; }
    }

    /// <summary>
    /// 租户Id接口
    /// </summary>
    public interface ITenantId
    {
        /// <summary>
        /// 租户Id（雪花id）
        /// </summary>
        public long? TenantId { get; set; }
    }

    /// <summary>
    /// 机构Id接口
    /// </summary>
    public interface IOrgId
    {
        /// <summary>
        /// 创建者部门Id（雪花id）
        /// </summary>
        public long? CreateOrgId { get; set; }
    }

    /// <summary>
    /// 框架实体基类Id
    /// </summary>
    public abstract class EntityBaseId : IEntity
    {
        /// <summary>
        /// 雪花Id
        /// </summary>
        [JsonConverter(typeof(ValueToStringConverter))]
        [SugarColumn(IsPrimaryKey = true, ColumnDescription = "主键")]
        public virtual long Id { get; set; }
    }

    /// <summary>
    /// 框架实体基类
    /// </summary>
    public abstract class EntityBase : EntityBaseId, IDeleted
    {

        /// <summary>
        /// 创建时间
        /// </summary>
        [SugarColumn(IsOnlyIgnoreUpdate = true, InsertServerTime =  true, ColumnDescription = "创建时间")]
        public virtual DateTime CreateTime { get; set; }

        /// <summary>
        /// 更新时间
        /// </summary>
        [SugarColumn(IsOnlyIgnoreInsert = true, UpdateServerTime = true, ColumnDescription = "更新时间")]
        public virtual DateTime? UpdateTime { get; set; }

        /// <summary>
        /// 创建者Id
        /// </summary>
        [JsonConverter(typeof(ValueToStringConverter))]
        [SugarColumn(IsOnlyIgnoreUpdate = true)]
        public virtual long? CreateUserId { get; set; }

        /// <summary>
        ///  创建者姓名
        /// </summary>
        [SugarColumn(IsOnlyIgnoreUpdate = true)]
        public virtual string? CreateUserName { get; set; } = string.Empty;

        /// <summary>
        /// 修改者Id
        /// </summary>
        [JsonConverter(typeof(ValueToStringConverter))]
        [SugarColumn(IsOnlyIgnoreInsert = true)]
        public virtual long? UpdateUserId { get; set; }

        /// <summary>
        /// 修改者姓名
        /// </summary>
        [SugarColumn(IsOnlyIgnoreInsert = true)]
        public virtual string? UpdateUserName { get; set; } = string.Empty;

        /// <inheritdoc/>
        public virtual DateTime? DeleteTime { get; set; }
    }

    /// <summary>
    /// 业务数据实体基类(数据权限)
    /// </summary>
    public abstract class EntityBaseData : EntityBase, IOrgId
    {
        /// <inheritdoc/>
        [JsonConverter(typeof(ValueToStringConverter))]
        public virtual long? CreateOrgId { get; set; }

        /// <summary>
        /// 创建者部门名称
        /// </summary>
        public virtual string? CreateOrgName { get; set; } = string.Empty;
    }

    /// <summary>
    /// 租户基类实体
    /// </summary>
    public abstract class EntityTenant : EntityBase, ITenantId
    {
        /// <inheritdoc/>
        [JsonConverter(typeof(ValueToStringConverter))]
        public virtual long? TenantId { get; set; }
    }

    /// <summary>
    /// 租户基类实体Id
    /// </summary>
    public abstract class EntityTenantId : EntityBaseId, ITenantId
    {
        /// <inheritdoc/>
        [JsonConverter(typeof(ValueToStringConverter))]
        public virtual long? TenantId { get; set; }
    }

    /// <summary>
    /// 实体种子数据接口
    /// </summary>
    /// <typeparam name="TEntity"></typeparam>
    public interface ISqlSugarEntitySeedData<TEntity>
        where TEntity : class, new()
    {
        /// <summary>
        /// 种子数据
        /// </summary>
        /// <returns></returns>
        IEnumerable<TEntity> HasData();
    }

    #endregion

    #region 枚举相关
    /// <summary>
    /// 验证状态
    /// </summary>
    public enum ValidEnum
    {
        /// <summary>
        /// 未验证
        /// </summary>
        [Description("未验证")] Validation = 0,
        /// <summary>
        /// 已验证
        /// </summary>
        [Description("已验证")] Authenticated = 1,
    }
    /// <summary>
    /// 验证状态
    /// </summary>
    public enum BooleanEnum
    {
        /// <summary>
        /// 否
        /// </summary>
        [Description("否")] No = 0,
        /// <summary>
        /// 是
        /// </summary>
        [Description("是")] Yes = 1,
    }
    /// <summary>
    /// YesNoEnum
    /// </summary>
    public enum YesNoEnum
    {
        /// <summary>
        /// 否
        /// </summary>
        [Description("否")] N = 0,
        /// <summary>
        /// 是
        /// </summary>
        [Description("是")] Y = 1,
    }
    /// <summary>
    /// 状态枚举带禁用
    /// </summary>
    public enum StatusEnum
    {
        /// <summary>
        /// 启用
        /// </summary>
        [Description("启用")] Enable = 0,
        /// <summary>
        /// 禁用
        /// </summary>
        [Description("禁用")] Disable = 1,
    }
    /// <summary>
    /// 登录枚举带禁用
    /// </summary>
    public enum SignStatusEnum
    {
        /// <summary>
        /// 正常
        /// </summary>
        [Description("正常")] Normal = 0,
        /// <summary>
        /// 已禁用
        /// </summary>
        [Description("已禁用")] Disable = 1,
        /// <summary>
        /// 已锁定
        /// </summary>
        [Description("已锁定")] Blockade = 2,
    }

    /// <summary>
    /// 用户注册类型枚举
    /// </summary>
    public enum SignUpTypeEnum
    {
        /// <summary>
        /// 账号密码
        /// </summary>
        [Description("账号密码")] Account = 0,
        /// <summary>
        /// 手机号
        /// </summary>
        [Description("手机号")] Phone = 1,
        /// <summary>
        /// 邮箱
        /// </summary>
        [Description("邮箱")] Email = 2,
    }
    /// <summary>
    /// 审核状态枚举
    /// </summary>
    public enum ApprovalStatusEnum
    {
        /// <summary>
        /// 待审核
        /// </summary>
        [Description("待审核")] Normal = 0,
        /// <summary>
        /// 审核中
        /// </summary>
        [Description("审核中")] Review = 1,
        /// <summary>
        /// 已批准
        /// </summary>
        [Description("已批准")] Approve = 2,
        /// <summary>
        /// 已驳回
        /// </summary>
        [Description("已驳回")] Reject = 3,
    }

    /// <summary>
    /// 统计方法枚举
    /// </summary>
    public enum StatisticalMethodEnum
    {
        /// <summary>
        /// 查看
        /// </summary>
        [Description("查看")] View = 0,
        /// <summary>
        /// 下载
        /// </summary>
        [Description("下载")] Download = 1,
        /// <summary>
        /// 分享
        /// </summary>
        [Description("分享")] Share = 2
    }

    /// <summary>
    /// 通用任务状态
    /// </summary>
    public enum CommonTaskStatus
    {
        /// <summary>
        /// 等待中
        /// </summary>
        [Description("等待中")] Waiting,
        /// <summary>
        /// 执行中
        /// </summary>
        [Description("执行中")] Running,
        /// <summary>
        /// 已完成
        /// </summary>
        [Description("已完成")] Finished,
        /// <summary>
        /// 已失败
        /// </summary>
        [Description("已失败")] Failure
    }


    /// <summary>
    /// 角色数据范围枚举
    /// </summary>
    public enum DataScopeEnum
    {
        /// <summary>
        /// 全部数据
        /// </summary>
        [Description("全部数据")] All = 1,

        /// <summary>
        /// 本部门及以下数据
        /// </summary>
        [Description("本部门及以下数据")] DeptChild = 2,

        /// <summary>
        /// 本部门数据
        /// </summary>
        [Description("本部门数据")] Dept = 3,

        /// <summary>
        /// 仅本人数据
        /// </summary>
        [Description("仅本人数据")] Self = 4,

        /// <summary>
        /// 自定义数据
        /// </summary>
        [Description("自定义数据")] Define = 5,
    }

    /// <summary>
    /// 系统菜单类型枚举
    /// </summary>
    public enum MenuTypeEnum
    {
        /// <summary>
        /// 目录
        /// </summary>
        [Description("目录")] Dir = 1,

        /// <summary>
        /// 菜单
        /// </summary>
        [Description("菜单")] Menu = 2,

        /// <summary>
        /// 按钮
        /// </summary>
        [Description("按钮")] Btn = 3,
    }

    /// <summary>
    /// 账号类型枚举
    /// </summary>
    public enum AccountTypeEnum
    {
        /// <summary>
        /// 超级管理员
        /// </summary>
        [Description("超级管理员")] SuperAdmin = 0,
        /// <summary>
        /// 系统管理员
        /// </summary>
        [Description("系统管理员")] SysAdmin = 1,
        /// <summary>
        /// 普通账号
        /// </summary>
        [Description("普通账号")] NormalUser = 2,
    }


    /// <summary>
    /// 性别枚举
    /// </summary>
    public enum SexEnum
    {
        /// <summary>
        /// 男
        /// </summary>
        [Description("男")] Man = 1,
        /// <summary>
        /// 女
        /// </summary>
        [Description("女")] Woman = 2,
        /// <summary>
        /// 未知
        /// </summary>
        [Description("未知")] Unknown = 3
    }
    #endregion

    #region 系统常量KEY
    /// <summary>
    /// 系统常量KEY
    /// </summary>
    public class Constants
    {
        public const string CachePrefixCaptchaImage = "CachePrefixCaptchaImage";
    }


    /// <summary>
    /// 平台配置
    /// </summary>
    public class CommonConst
    {
        /// <summary>
        /// 演示环境开关
        /// </summary>
        public const string SysDemoEnv = "sys_demo";

        /// <summary>
        /// 默认密码
        /// </summary>
        public const string SysPassword = "sys_password";

        /// <summary>
        /// 登录二次验证
        /// </summary>
        public const string SysSecondVer = "sys_second_ver";

        /// <summary>
        /// 开启图形验证码
        /// </summary>
        public const string SysCaptcha = "sys_captcha";

        /// <summary>
        /// 开启水印
        /// </summary>
        public const string SysWatermark = "sys_watermark";

        /// <summary>
        /// 开启操作日志
        /// </summary>
        public const string SysOpLog = "sys_oplog";

        /// <summary>
        /// Token过期时间
        /// </summary>
        public const string SysTokenExpire = "sys_token_expire";

        /// <summary>
        /// RefreshToken过期时间
        /// </summary>
        public const string SysRefreshTokenExpire = "sys_refresh_token_expire";

        /// <summary>
        /// 单用户登录
        /// </summary>
        public const string SysSingleLogin = "sys_single_login";

        /// <summary>
        /// 系统管理员角色编码
        /// </summary>
        public const string SysAdminRole = "sys_admin";

        /// <summary>
        /// 开启全局脱敏处理（默认不开启）
        /// </summary>
        public static bool SysSensitiveDetection = false;
    }
    #endregion
}
