﻿using Microsoft.IdentityModel.Tokens;
using System;
using System.Collections.Generic;
using System.IdentityModel.Tokens.Jwt;
using System.Linq;
using System.Security.Claims;
using System.Security.Principal;
using System.Text;
using System.Threading.Tasks;

namespace Core.Extensions
{
    public static class ClaimsExtensions
    {
        /// <summary>
        /// 用户IdKEY
        /// </summary>
        public const string UserId = "Id";
        /// <summary>
        /// 用户名KEY
        /// </summary>
        public const string UserName = "Name";
        /// <summary>
        /// 用户头像KEY
        /// </summary>
        public const string UserAvatar = "Avatar";
        /// <summary>
        /// 用户角色KEY
        /// </summary>
        public const string RoleName = "Role";
        /// <summary>
        /// 登录凭证KEY
        /// </summary>
        public const string Identifier = "Identifier";
        /// <summary>
        /// 用户机构Id
        /// </summary>
        public const string OrgId = "OrgId";
        /// <summary>
        /// 租户KEY
        /// </summary>
        public const string TenantId = "TenantId";
        /// <summary>
        /// 超级管理员KEY
        /// </summary>
        public const string Super = "Super";

        /// <summary>
        /// 从Claims中获取指定Value
        /// </summary>
        /// <param name="claims"></param>
        /// <param name="claimTypes"></param>
        /// <returns></returns>
        public static string GetClaimsValue(this IEnumerable<Claim> claims, string claimTypes)
        {
            return claims.FirstOrDefault(c => c.Type == claimTypes)?.Value ?? string.Empty;
        }

        /// <summary>
        /// 从Claims中获取指定Value列表
        /// </summary>
        /// <param name="claims"></param>
        /// <param name="claimTypes"></param>
        /// <returns></returns>
        public static string[] GetClaimsValueList(this IEnumerable<Claim> claims, string claimTypes)
        {
            return claims.Where(p => p.Type == claimTypes).Select(p => p.Value).ToArray();
        }

        /// <summary>
        /// 获取用户ID
        /// </summary>
        /// <param name="claims"></param>

        public static long? GetUserId(this IEnumerable<Claim> claims)
        {
            if (long.TryParse(claims.GetClaimsValue(UserId), out var userId))
            {
                return userId;
            }
            else
            {
                return null;
            }
        }
        /// <summary>
        /// 获取用户名
        /// </summary>
        /// <param name="claims"></param>
        public static string GetUserName(this IEnumerable<Claim> claims) => claims.GetClaimsValue(UserName);
        /// <summary>
        /// 获取用户头像
        /// </summary>
        /// <param name="claims"></param>
        public static string GetUserAvatar(this IEnumerable<Claim> claims) => claims.GetClaimsValue(UserAvatar);
        /// <summary>
        /// 获取角色名称
        /// </summary>
        /// <param name="claims"></param>
        public static string GetRoleName(this IEnumerable<Claim> claims) => claims.GetClaimsValue(RoleName);
        /// <summary>
        /// 获取认证ID
        /// </summary>
        /// <param name="claims"></param>
        public static string GetIdentifier(this IEnumerable<Claim> claims) => claims.GetClaimsValue(Identifier);
        /// <summary>
        /// 获取租户ID
        /// </summary>
        /// <param name="claims"></param>
        public static long? GetTenantId(this IEnumerable<Claim> claims)
        {
            if (long.TryParse(claims.GetClaimsValue(TenantId), out var tenantId))
            {
                return tenantId;
            }
            else
            {
                return null;
            }
        }
        /// <summary>
        /// 获取租户ID
        /// </summary>
        /// <param name="claims"></param>
        public static long? GetOrgId(this IEnumerable<Claim> claims)
        {
            if (long.TryParse(claims.GetClaimsValue(OrgId), out var orgId))
            {
                return orgId;
            }
            else
            {
                return null;
            }
        }
        /// <summary>
        /// 获取用户是否为超级管理员
        /// </summary>
        /// <param name="claims"></param>
        public static bool GetSuper(this IEnumerable<Claim> claims)
        {
            if (bool.TryParse(claims.GetClaimsValue(Super), out var isSuper))
            {
                return isSuper;
            }
            else
            {
                return false;
            }
        }
        /// <summary>
        /// 基于 claims 创建 jwt token
        /// </summary>
        /// <param name="claims"></param>
        /// <param name="secret"></param>
        /// <param name="issuer"></param>
        /// <param name="audience"></param>
        /// <param name="expiration">如果设置为 int.MaxValue，则默认为10年</param>
        /// <param name="expires"></param>
        /// <returns></returns>
        public static string CreateJwtToken(this IEnumerable<Claim> claims, string secret, string issuer, string audience, int expiration, out DateTime expires)
        {
            expires = expiration.Equals(int.MaxValue) ? DateTime.Now.AddYears(10) : DateTime.Now.AddSeconds(expiration);
            var key = new SymmetricSecurityKey(secret.GetBytes());
            var credentials = new SigningCredentials(key, SecurityAlgorithms.HmacSha256);
            var jwtAccessToken = new JwtSecurityToken(issuer, audience, claims,
                expires: expires, signingCredentials: credentials,
                notBefore: DateTime.Now);
            return new JwtSecurityTokenHandler().WriteToken(jwtAccessToken);
        }

    }
}
